# frozen_string_literal: true

require "is/extension"
require "is/stateful"

require_relative "../core/event/callable"

module Is
  # [public] Adds events and callbacks to an object.
  #
  module Eventable
    extend Is::Extension

    applies do |extended:|
      # [public] The current eventable pipeline.
      #
      state :events, default: Core::Event::Callable.new(extended ? singleton_class : self)
    end

    extends :definition do
      # [public] Defines one or more events by name.
      #
      def event(*defined_events)
        events.register(*defined_events)
      end

      # [public] Defines a callback to be called during, before, and/or after the given events.
      #
      def callback(*during, before: [], after: [], method: nil, context: nil, &block)
        events.callback(*during, before: before, after: after, method: method, context: context, &block)
      end
    end

    extends :implementation, dependencies: [Is::Stateful] do
      # [public] Calls callbacks for the given event with the given arguments.
      #
      def performing(event, ...)
        @events.performing(self, event, ...)
      end

      # [public] Calls before callbacks for the given event with the given arguments.
      #
      def starting(event, ...)
        @events.starting(self, event, ...)
      end

      # [public] Calls after callbacks for the given event with the given arguments.
      #
      def finished(event, ...)
        @events.finished(self, event, ...)
      end
    end
  end
end
